@@ -96,7 +96,8 @@ class AthenaPartition:
             partitions_tracked += 1
         return partitions_list
 
-    def generate_sql(self, table, partitions_list, s3_path=None, athena_client=None, output_location=None):
+    def generate_sql(self, table, partitions_list, s3_path=None, athena_client=None, output_location=None,
+                     athena_s3_client=None):
         """
                 :param table: The athena table partitions are being loaded for
                 :param partitions_list: The list of all the partitions to generate the alter table query for
@@ -104,6 +105,8 @@ class AthenaPartition:
                                 the toplevel of the bucket
                 :param athena_client: The current Athena Client, optional, will be used to list partitions already loaded
                 :param output_location: Output location of queries parsed into athena_client
+                :param athena_s3_client: The s3 client that matches with the athena client passed in to be able to
+                access the queries produced by it if in different account to default
                 :return: A list of queries that can be run to add all partitions
                 """
         s3_path = table if s3_path is None else s3_path
@@ -117,10 +120,12 @@ class AthenaPartition:
                                             output_location=output_location)
 
             athena_client.wait_for_completion()
-            fm = FileDoggo(os.path.join(query.arguments["output_location"], "{}.txt".format(query.id)), mode='rb', compression=None)
+            fm = FileDoggo(os.path.join(query.arguments["output_location"], "{}.txt".format(query.id)), mode='rb',
+                           compression=None, client=athena_s3_client)
             with fm as f:
                 athena_list = f.read().decode().splitlines()
-            athena_list = [{'Prefix': '{table}/{partition}/'.format(table=table, partition=partition)} for partition in athena_list]
+            athena_list = [{'Prefix': '{table}/{partition}/'.format(table=table, partition=partition)} for partition in
+                           athena_list]
             partitions_list = [i for i in partitions_list if i not in athena_list]
 
         partition_string_list = []
@@ -153,7 +158,8 @@ class AthenaPartition:
 
         return full_query_string_list
 
-    def get_sql(self, table, partition_depth=None, s3_path=None, athena_client=None, output_location=None):
+    def get_sql(self, table, partition_depth=None, s3_path=None, athena_client=None, output_location=None,
+                athena_s3_client=None):
         """
         :param table: The athena table partitions are being loaded for
         :param partition_depth: The number of partition levels to look for, optional param should be specified if
@@ -162,9 +168,11 @@ class AthenaPartition:
                         level of the bucket
         :param athena_client: The current Athena Client, optional, will be used to list partitions already loaded
         :param output_location: Output location of queries parsed into athena_client
+        :param athena_s3_client: The s3 client that matches with the athena client passed in to be able to
+                access the queries produced by it if in different account to default
         :return: A list of queries that can be run to add all partitions, each query is limited to 10 partitions
         """
 
         partitions_list = self.list_partitions(table, partition_depth, s3_path)
 
-        return self.generate_sql(table, partitions_list, s3_path, athena_client, output_location)
+        return self.generate_sql(table, partitions_list, s3_path, athena_client, output_location, athena_s3_client)
