@@ -64,7 +64,7 @@ S3fsMixin = DoggoFileSystem
 class BaseCachedClass:
     validation_mode = False
 
-    def __init__(self, cache_path, logger=logging.getLogger("newtools.cached_query") ):
+    def __init__(self, cache_path, logger=logging.getLogger("newtools.cached_query")):
         self.cache_path = cache_path
         self.dfs = DoggoFileSystem()
         self._logger = logger
@@ -136,6 +136,7 @@ class BaseCachedClass:
 
 class BaseCachedQuery(BaseCachedClass):
     _archive_path = None
+    _redshift_checks = False
 
     wait_period = 30
     time_out_seconds = 1800
@@ -166,7 +167,6 @@ class BaseCachedQuery(BaseCachedClass):
                 sql_paths = [sql_paths]
             self._sql_paths = sql_paths + self._sql_paths
 
-
         self._archive_path = sql_archive_path
         self._aws_lock = DynamoDogLock if dynamodb_lock else DoggoLock
 
@@ -361,7 +361,7 @@ class BaseCachedQuery(BaseCachedClass):
                                         replacement_dict={} if replacement_dict is None else replacement_dict)
 
                     # In the S3 case, redshift files are saved with a prefix and need to be renamed
-                    if output_file.startswith("s3://"):
+                    if self._redshift_checks and output_file.startswith("s3://"):
                         if not self.dfs.exists(output_file):
                             files = self.dfs.glob(output_file + "**")
                             if len(files) == 0:
@@ -410,6 +410,7 @@ class CachedPep249Query(BaseCachedQuery):
 
     """
     __sql = None
+    _redshift_checks = True
 
     def __init__(self, pep_249_obj, params=None, cache_path="", sql_archive_path=None, sql_paths=None, gzip=True,
                  dynamodb_lock=True, logger=logging.getLogger("newtools.cached_query")):
@@ -470,6 +471,9 @@ class CachedAthenaQuery(BaseCachedQuery):
     """
     The cached AthenaQuery will execute queries against AWS Athena and cache results in csv.gz format.
 
+    It takes single additiona arguments, queue_queries, which defaults to False but will queue the queries
+    before executing them if set to true.
+
     Two special parameters must be passed to connect to Athena:
 
     * **aws_region** - the AWS region
@@ -500,6 +504,14 @@ class CachedAthenaQuery(BaseCachedQuery):
     """
     __ac = None
 
+    def __init__(self, params=None, cache_path="", sql_archive_path=None, sql_paths=None, gzip=True,
+                 dynamodb_lock=True, logger=logging.getLogger("newtools.cached_query"),
+                 queue_queries=False):
+        super().__init__(params, cache_path, sql_archive_path, sql_paths, gzip, dynamodb_lock, logger)
+
+        self._results = dict()
+        self.queue_queries = queue_queries
+
     @property
     def _ac(self):
         """
@@ -559,22 +571,32 @@ class CachedAthenaQuery(BaseCachedQuery):
             self._archive_query(actual_query, query_parameters, archive_query_path)
 
         # Now now the query on Athena
-        res = self._ac.add_query(
+        self._results[output_file] = self._ac.add_query(
             sql=actual_query,
             name="unload {}".format(output_file),
             output_location=self.cache_path)
+
+        if not self.queue_queries:
+            self.wait_for_completion()
+
+    def wait_for_completion(self):
+
         self._ac.wait_for_completion()
 
-        actual_file = S3Location(self.cache_path).join(res.id + '.csv')
+        for output_file in set(self._results.keys()):
 
-        # gzip the output
-        if self._gzip:
-            gz_file = output_file
-            with self.dfs.open(actual_file, 'rb') as f_in:
-                with self.dfs.open(gz_file, 'wb') as f_out:
-                    with gz.open(f_out, 'wb') as gz_out:
-                        shutil.copyfileobj(f_in, gz_out)
+            res = self._results.pop(output_file)
 
-        # delete the metadata file
-        self.dfs.rm(actual_file)
-        self.dfs.rm(actual_file + '.metadata')
+            actual_file = S3Location(self.cache_path).join(res.id + '.csv')
+
+            # gzip the output
+            if self._gzip:
+                gz_file = output_file
+                with self.dfs.open(actual_file, 'rb') as f_in:
+                    with self.dfs.open(gz_file, 'wb') as f_out:
+                        with gz.open(f_out, 'wb') as gz_out:
+                            shutil.copyfileobj(f_in, gz_out)
+
+            # delete the metadata file
+            self.dfs.rm(actual_file)
+            self.dfs.rm(actual_file + '.metadata')
