@@ -12,6 +12,9 @@ from newtools import optional_imports
 
 
 class BaseTest(TestCase):
+    """
+    A basic test class that compares the existence of files in different locations
+    """
     create_test_data = False
     test_dir = os.path.join(os.path.dirname(__file__), "test_data")
     tempdir = os.path.join(test_dir, "temp")
@@ -29,7 +32,18 @@ class BaseTest(TestCase):
             self.setUp()
 
     def tearDown(self):
-        shutil.rmtree(self.tempdir)
+        shutil.rmtree(self.tempdir, )
+
+    def compare_file(self, actual, expected):
+        print(actual)
+        if self.create_test_data:
+            shutil.copyfile(actual, expected)
+
+        with open(expected) as expected_file:
+            with open(actual, "r") as actual_file:
+                self.assertEqual(expected_file.readlines(),
+                                 actual_file.readlines(),
+                                 "actual file(right) is not as expected (left)")
 
     def compare_files(self, expected_path):
         for file in glob.glob(os.path.join(self.tempdir, "*")):
@@ -42,15 +56,7 @@ class BaseTest(TestCase):
                     with open(file, "wb") as f_out:
                         f_out.write(f_in.read())
 
-            print(file)
-            if self.create_test_data:
-                shutil.copyfile(file, os.path.join(expected_path, os.path.split(file)[1]))
-            with open(os.path.join(expected_path,
-                                   os.path.split(file)[1])) as expected_file:
-                with open(file, "r") as actual_file:
-                    self.assertEqual(expected_file.readlines(),
-                                     actual_file.readlines(),
-                                     "actual file(right) is not as expected (left)")
+            self.compare_file(file, os.path.join(expected_path, os.path.split(file)[1]))
 
 
 class TestBase(BaseTest):
