@@ -2,22 +2,26 @@
 # -----------------------------------------
 #  This code is licensed under MIT license (see license.txt for details)
 
-import unittest
-import os
-import pyarrow as pa
-
-from time import sleep
-import multiprocessing
-from random import random
 import gzip
+import multiprocessing
+import os
+import shutil
+import unittest
+from random import random
+from time import sleep
 
-from newtools.optional_imports import pandas as pd
-from newtools.optional_imports import numpy as np
-from newtools.optional_imports import boto3
+import pyarrow as pa
+from pandas.testing import assert_frame_equal
+from s3fs import S3FileSystem
 
+from newtools import CSVDoggo
 from newtools import PandasDoggo, FileDoggo, S3Location, DoggoFileSystem, DoggoLock, DoggoWait, DynamoDogLock
+from newtools.optional_imports import boto3
+from newtools.optional_imports import numpy as np
+from newtools.optional_imports import pandas as pd
 from .base_test import BaseTest
 
+s3_bucket = "newtools-testing"
 
 class TestDoggos(unittest.TestCase):
 
@@ -25,6 +29,21 @@ class TestDoggos(unittest.TestCase):
     def setUpClass(cls):
         cls.base_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'test_data', 'pandas_doggo_tests')
 
+        # Upload test data to s3
+        cls.s3_base_path = f"s3://{s3_bucket}/generic/"
+        cls.s3 = S3FileSystem()
+        for file in cls.s3.glob(f"{cls.s3_base_path}*"):
+            cls.s3.rm(file)
+
+        cls.s3.put(lpath=os.path.join(cls.base_path, "csv", "generic"),
+                   rpath=cls.s3_base_path,
+                   recursive=True)
+
+    @classmethod
+    def tearDownClass(cls):
+        for file in cls.s3.glob(f"{cls.s3_base_path}*"):
+            cls.s3.rm(file)
+
     def test_read_csv_local(self):
         """read a local csv"""
 
@@ -235,7 +254,7 @@ class TestDoggos(unittest.TestCase):
         expected_df = pd.DataFrame({'e-mail': {0: 'n@gmail.com', 1: 'p@gmail.com', 2: 'h@gmail.com', 3: 's@gmail.com',
                                                4: 'l@gmail.com', 5: 'v@gmail.com', 6: None},
                                     'number': {0: 0, 1: 1, 2: 0, 3: 0, 4: 1, 5: 0, 6: 0}})
-        pth = S3Location('deductive-tools-testing').join('generic', 'email_test.csv')
+        pth = S3Location('newtools-testing').join('generic', 'email_test.csv')
         session = boto3.Session()
         fh = PandasDoggo(boto_session=session)
         df = fh.load(pth)
@@ -245,8 +264,8 @@ class TestDoggos(unittest.TestCase):
         df = pd.DataFrame({'e-mail': {0: 'n@gmail.com', 1: 'p@gmail.com', 2: 'h@gmail.com', 3: 's@gmail.com',
                                       4: 'l@gmail.com', 5: 'v@gmail.com', 6: None},
                            'number': {0: 0, 1: 1, 2: 0, 3: 0, 4: 1, 5: 0, 6: 0}})
-        pth = S3Location('deductive-tools-testing').join('generic', 'email_test_copy.csv')
-        expected_pth = S3Location('deductive-tools-testing').join('generic', 'email_test.csv')
+        pth = S3Location('newtools-testing').join('generic', 'email_test_copy.csv')
+        expected_pth = S3Location('newtools-testing').join('generic', 'email_test.csv')
         fh = PandasDoggo()
         fh.save(df, pth, index=None)
         with FileDoggo(pth) as p, FileDoggo(expected_pth) as p2:
@@ -260,7 +279,7 @@ class TestDoggos(unittest.TestCase):
         df = pd.DataFrame({'e-mail': {0: 'n@gmail.com', 1: 'p@gmail.com', 2: 'h@gmail.com', 3: 's@gmail.com',
                                       4: 'l@gmail.com', 5: 'v@gmail.com', 6: None},
                            'number': {0: 0, 1: 1, 2: 0, 3: 0, 4: 1, 5: 0, 6: 0}})
-        pth = S3Location('deductive-tools-testing').join('generic', 'email_test_copy.csv.gz')
+        pth = S3Location('newtools-testing').join('generic', 'email_test_copy.csv.gz')
 
         fh = PandasDoggo()
 
@@ -299,7 +318,7 @@ class TestDoggos(unittest.TestCase):
         df = pd.DataFrame({'e-mail': {0: 'n@gmail.com', 1: 'p@gmail.com', 2: 'h@gmail.com', 3: 's@gmail.com',
                                       4: 'l@gmail.com', 5: 'v@gmail.com', 6: None},
                            'number': {0: 0, 1: 1, 2: 0, 3: 0, 4: 1, 5: 0, 6: 0}})
-        pth = S3Location('deductive-tools-testing').join('generic', 'email_test_copy.parquet')
+        pth = S3Location('newtools-testing').join('generic', 'email_test_copy.parquet')
 
         fh = PandasDoggo()
 
@@ -459,7 +478,7 @@ class BaseLockTest(BaseTest):
             file = os.path.join(self.tempdir, 'test_lock_multiprocess.tmp')
             pool = multiprocessing.Pool()
 
-            for _ in range(0, 100):
+            for _ in range(0, 10):
                 pool.apply_async(check_lock,
                                  (file,))
 
@@ -468,6 +487,7 @@ class BaseLockTest(BaseTest):
 
 
 class TestDoggoLock(BaseLockTest):
+    tempdir = 's3://newtools-tests-data/locks/'
     LockClass = DoggoLock
 
 
@@ -480,3 +500,128 @@ class TestDynamoDogLock(BaseLockTest):
         file = os.path.join(self.tempdir, 'test_lock_ages.tmp')
         with self.LockClass(file, wait_period=1, time_out_seconds=2, maximum_age=5):
             pass
+
+
+class TestPartitionDoggo(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        cls.df = pd.DataFrame({"col1": [1, 2, 3, 4, 5, 1, "4", 3, 4, "4", "4", 4, 2, 3, "4"],
+                               "col2": [6, 7, 8, 9, 10, 5, 6, 7, 5, "10", 7, 6, 8, 7, 6],
+                               "col3": [11, 12, 13, 14, 15, "a", "b", "y", 1, 3, 15, 66, 77, 13, 16],
+                               "col4": ["sda", "", 16, "17", 18, "14", 12, 13, 14, 5, 6, 45, 34, 76, 87]}).astype(
+            {"col4": str, "col2": int, "col3": str})
+        try:
+            os.mkdir(os.path.join(os.path.dirname(__file__), "test_data/athena_partition/output"))
+        except FileExistsError:
+            pass
+        cls.base_path = os.path.join(os.path.dirname(__file__), "test_data/athena_partition/output")
+
+    def test_csv_out(self):
+        expected_data = os.path.join(os.path.dirname(__file__), "test_data/athena_partition/expected_csv.csv")
+        for file in os.listdir(self.base_path):
+            if not file.startswith('.'):
+                file_path = os.path.join(self.base_path, file)
+                shutil.rmtree(file_path)
+
+        partition_cat = ["col1"]
+        cols_keep = ["col2", "col4"]
+        output_data_path = os.path.join(self.base_path, "col1=4")
+        PandasDoggo().save_partitioned(df=self.df[partition_cat + cols_keep].astype(dict(zip(partition_cat, [int]))),
+                                       partition_columns=partition_cat,
+                                       base_path=self.base_path,
+                                       name="/test_name_",
+                                       suffix=".csv",
+                                       index=False)
+        self.assertEqual(1, len([name for name in os.listdir(output_data_path)]))
+        csv = CSVDoggo()
+        expected = csv.load_df(expected_data)
+        output = csv.load_df(os.path.join(output_data_path, *os.listdir(output_data_path)))
+
+        assert_frame_equal(expected, output, check_like=True)
+
+        for file in os.listdir(self.base_path):
+            if not file.startswith('.'):
+                file_path = os.path.join(self.base_path, file)
+                shutil.rmtree(file_path)
+
+    def test_pq_out(self):
+        expected_data = os.path.join(os.path.dirname(__file__), "test_data/athena_partition/expected_parquet.parquet")
+        for file in os.listdir(self.base_path):
+            if not file.startswith('.'):
+                file_path = os.path.join(self.base_path, file)
+                shutil.rmtree(file_path)
+
+        partition_cat = ["col1"]
+        cols_keep = ["col2", "col4"]
+        output_data_path = os.path.join(self.base_path, "col1=4")
+        PandasDoggo().save_partitioned(df=self.df[partition_cat + cols_keep].astype(dict(zip(partition_cat, [int]))),
+                                       partition_columns=partition_cat,
+                                       base_path=self.base_path,
+                                       name="/test_name_",
+                                       suffix=".parquet",
+                                       index=False)
+
+        dog = PandasDoggo()
+        expected = dog.load_parquet(expected_data)
+        output = dog.load_parquet(os.path.join(output_data_path, *os.listdir(output_data_path)))
+        assert_frame_equal(expected, output, check_like=True)
+
+        for file in os.listdir(self.base_path):
+            if not file.startswith('.'):
+                file_path = os.path.join(self.base_path, file)
+                shutil.rmtree(file_path)
+
+    def test_multi_part(self):
+        expected_data = os.path.join(os.path.dirname(__file__), "test_data/athena_partition/expected_multi_csv.csv")
+        for file in os.listdir(self.base_path):
+            if not file.startswith('.'):
+                file_path = os.path.join(self.base_path, file)
+                shutil.rmtree(file_path)
+
+        csv = CSVDoggo(base_path=self.base_path)
+        partition_cat = ["col1", "col2"]
+        output_data_path = os.path.join(self.base_path, "col1=4/col2=6")
+        PandasDoggo().save_partitioned(df=self.df.astype(dict(zip(partition_cat, [int, int]))),
+                                       partition_columns=partition_cat,
+                                       base_path=self.base_path,
+                                       name="/test_name_",
+                                       suffix=".parquet",
+                                       index=False)
+
+        self.assertEqual(5, len([name for name in os.listdir(os.path.join(self.base_path, "col1=4"))]))
+        dog = PandasDoggo()
+        expected = dog.load(expected_data, dtype=object)
+        output = dog.load(os.path.join(output_data_path, *os.listdir(output_data_path)))
+
+        assert_frame_equal(expected, output, check_like=True)
+
+        for file in os.listdir(self.base_path):
+            if not file.startswith('.'):
+                file_path = os.path.join(self.base_path, file)
+                shutil.rmtree(file_path)
+
+    def test_expected_errors(self):
+
+        with self.assertRaises(ValueError):
+            PandasDoggo().save_partitioned(df=self.df.astype(dict(zip(["col4"], [int, int]))),
+                                           partition_columns=["col4"],
+                                           base_path=self.base_path,
+                                           name="/test_name_",
+                                           suffix=".parquet",
+                                           index=False)
+
+        with self.assertRaises(KeyError):
+            PandasDoggo().save_partitioned(df=self.df.astype(dict(zip(["col1", "col7"], [int, int]))),
+                                           partition_columns=["col1"],
+                                           base_path=self.base_path,
+                                           name="/test_name_",
+                                           suffix=".parquet",
+                                           index=False)
+
+        with self.assertRaises(ValueError):
+            PandasDoggo().save_partitioned(df=self.df.astype(dict(zip(["col3"], [int, int]))),
+                                           partition_columns=["col3"],
+                                           base_path=self.base_path,
+                                           name="/test_name_",
+                                           suffix=".parquet",
+                                           index=False)
