@@ -121,7 +121,7 @@ or last_name = 'Chapman';
 
     def test_log_to_s3(self):
 
-        s3_archive = S3Location('deductive-tools-testing/test_sql_archive/sql_log.log')
+        s3_archive = S3Location('newtools-testing/test_sql_archive/sql_log.log')
 
         sql = SqlClient(self.connection)
         sql.execute_query(os.path.join(self.base_path, "query_file.sql"),
